

def urljoin(*args):
    """
    Joins given arguments into an url. Trailing but not leading slashes are
    stripped for each argument.
    """

    return "/".join(map(lambda x: str(x).rstrip('/'), args))



import os
from django.conf import settings as S
from django.templatetags.static import static
from django.urls import reverse
from django.shortcuts import render
from django.template.loader import render_to_string
from django.http import HttpResponse
from .. import utils

import logging


logging.basicConfig(
    level = logging.DEBUG if S.DEBUG else logging.INFO,
    format = '%(levelname)s:%(name)s: %(message)s',
)
logger = logging.getLogger(__name__)

class Mixin(object):
    """Base mixin class offered by djpub
    
    Every djpub compatible view should inherit this mixin class.

    The following attributes must be overriden:
        * djpub_install_prefix
    """
    
    _djpub_active_theme_dir_name = 'active'
    djpub_install_prefix = None     # Must override

    def __init__(self, **kwargs):
        super(Mixin, self).__init__(**kwargs)
    
    def djpub_get_install_path_relative(self):
        """Return relative path from install prefix
        
        Replaces dots ('.') with os.path.sep
        
        Returns:
            str: relative path
        """

        return os.path.join(self.djpub_install_prefix.replace('.', os.path.sep), self._djpub_active_theme_dir_name)
    
    def djpub_template(self, relative_path):
        """Convert template relative path to actual template path
        
        Args:
            relative_path (str): template path relative to your package
        
        Returns:
            str: Actual relative path to the template root
        """
        return os.path.join(self.djpub_get_install_path_relative(), relative_path)

    def djpub_templates(self, relative_paths):
        """Convert template relative paths to actual template paths
        
        Args:
            relative_paths (list): List of template paths relative to your package
        
        Returns:
            list: A new list of templates paths relative to the template root
        """
        full_paths = []
        for relative_path in relative_paths:
            full_paths.append(self.djpub_template(relative_path))
        return full_paths
    
    def djpub_static(self, path):
        return static(utils.urljoin(self.djpub_get_install_path_relative(), path))
    
    def djpub_url(self, path):
        return reverse(utils.urljoin(self.djpub_get_install_path_relative(), path))

    
    def djpub_get_new_context(self):
        """Return a context already filled with required djpub context data
        
        Must use this method to create new context.
        
        Returns:
            dict: djpub context
        """
        djpub_ctx = {
            'djpub_theme_path': self.djpub_get_install_path_relative(),
            'djpub_template': self.djpub_template,
            'djpub_templates': self.djpub_templates,
            'static': self.djpub_static,
            'url': self.djpub_url,
        }
        return djpub_ctx

    def djpub_add_context_mandatory(self, ctx):
        """Add required djpub context to an existing context inplace.
        
        Args:
            ctx (dict): existing context
        """
        ctx.update(self.djpub_get_new_context())
    
    def djpub_render(self, request, template, ctx):
        """Render and return response handling djpub theme paths
        
        Args:
            request (HttpRequest): the request object
            template (str or list): one template or multiple templates
            ctx (dict): the context
        
        Returns:
            [type]: [description]
        """
        if isinstance(template, str):
            return render(request, self.djpub_template(template), ctx)
        else:
            return render(request, self.djpub_templates(template), ctx)

    def djpub_save_and_render(self, request, template, ctx, html_file):
        """Render and return response handling djpub theme paths
        
        Args:
            request (HttpRequest): the request object
            template (str or list): one template or multiple templates
            ctx (dict): the context
        
        Returns:
            [HttpResponse]: response
        """
        if isinstance(template, str):
            html = render_to_string(self.djpub_template(template), context=ctx, request=request)
        else:
            html = render_to_string(self.djpub_templates(template), context=ctx, request=request)
        try:
            with open(html_file, 'w', encoding='utf-8') as f:
                f.write(html)
        except:
            logger.exception("Failed to write file: " + html_file)
        return HttpResponse(html, content_type="text/html")






# -*- coding: utf-8 -*-

"""Common views for djpub


-------------------------------------------------------------------
Copyright: Md. Jahidul Hamid <jahidulhamid@yahoo.com>

License: [BSD](http://www.opensource.org/licenses/bsd-license.php)
-------------------------------------------------------------------
"""

from django.http import HttpResponse
from django.views import View
from django.utils.module_loading import import_string

class BaseView(View):
    """Base view to be inherited by other views
    """
    pass



from django.urls import path
import djpub
from .views import HomeView, PostView

class UrlPatterns(djpub.urls.UrlPatterns):
    """A class named UrlPatterns must be avaiable directly under djpub_view
    
    class path: [any_leading_path].djpub_view.UrlPatterns
    """

    def get_urlpatterns(self):
        """Return the url patterns handled by this view package
        
        Returns:
            list: django url patterns
        """
        urlpatterns = [
            path('', HomeView.as_view()),
            path('post/', PostView.as_view()),
        ]
        return urlpatterns

    def get_desired_position_index(self):
        """Return the index where these url patterns will be inserted.
        
        Returns:
            int: Desired position index
        """
        return 0



from django.views import View
from django.http import HttpResponse

class HomeView(View):

    def get(self, request):
        return HttpResponse(f'This is home')

class PostView(View):

    def get(self, request):
        return HttpResponse(f'This is a blog post')


# -*- coding: utf-8 -*-

"""URL related functions and classes

-------------------------------------------------------------------
Copyright: Md. Jahidul Hamid <jahidulhamid@yahoo.com>

License: [BSD](http://www.opensource.org/licenses/bsd-license.php)
-------------------------------------------------------------------
"""


import os
import glob
import logging
import pkgutil
from django.urls import path
from django.utils.module_loading import import_string
from django.conf import settings as S


logging.basicConfig(
    level = logging.DEBUG if S.DEBUG else logging.INFO,
    format = '%(levelname)s:%(name)s: %(message)s',
)
logger = logging.getLogger(__name__)


DJPUB_URL_CONF_NAME = 'djpub_urls'


class UrlPatterns(object):
    """Class to be inherited by djpub UrlPatterns in djpub_view
    
    Provides a get_urlpatterns() method which is called to get the url patterns.
    """

    current_djpub_view_dotted_path = '' # do not modify this variable in subclasses, only use

    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)
    
    def get_urlpatterns(self):
        """Return the url_patterns list that will be handled by the corresponding views

        Raises:
            NotImplementedError: This method must be implemented

        Returns:
            list: Should return a list of django URLPatterns
        """
        raise NotImplementedError(f"This method '{UrlPatterns.get_urlpatterns.__name__}' must be overriden in {self._get_current_djpub_view_dotted_path()}.{self.__class__.__name__}")
    
    def _get_urlpatterns(self):
        """Internal method. Do not override.

        Returns:
            list: List of django URLPatterns
        """
        return self.get_urlpatterns()
    
    def get_desired_position_index(self):
        """Return the desired position index where these url patterns should be inserted.
        
        0 will try to add these urlpatterns at the beginning.

        Multiple djpub_views using the same index will stay together but the order is not strictly defined.
        
        With current implementation the order is reverse alphabetic for same index patterns - [30/05/2019].
        
        Raises:
            NotImplementedError: This method must be implemented
        
        Returns:
            (int): A number designating the desired postion where these urlpatterns need to be inserted.
        """
        raise NotImplementedError(f"This method '{UrlPatterns.get_desired_position_index.__name__}' must be overriden in {self._get_current_djpub_view_dotted_path()}.{self.__class__.__name__}")
    
    def _get_desired_position_index(self):
        """Internal method. Do not override.
        
        Raises exception for invalid index
        
        Returns:
            int: position index
        """
        index = self.get_desired_position_index()
        if not isinstance(index, int):
            raise ValueError(f"Expected a int value from {self._get_current_djpub_view_dotted_path()}.{self.__class__.__name__}.{UrlPatterns.get_desired_position_index.__name__}, but {type(index)} is provided")
        if index < 0:
            raise ValueError(f"Position index can not be less than 0 in {self._get_current_djpub_view_dotted_path()}.{self.__class__.__name__}.{UrlPatterns.get_desired_position_index.__name__}")
        return index

    def get_current_djpub_view_dotted_path(self):
        """Return current_djpub_view_dotted_path if exists, otherwise None

        Do not override this method in subclasses.
        
        Returns:
            str: current djpub view dotted path or None
        """
        return self.current_djpub_view_dotted_path

    def _get_current_djpub_view_dotted_path(self):
        current_djpub_view_dotted_path = self.get_current_djpub_view_dotted_path()
        return current_djpub_view_dotted_path if current_djpub_view_dotted_path else '[current_djpub_view_dotted_path]'


def get_dotted_path(full_path, root_path):
    return full_path.replace(root_path, '').strip(os.path.sep).replace(os.path.sep, '.')


def get_urlpatterns(paths, root_path, exclude=None):
    """Search all packages in paths and return a list of urlpatterns from djpub urlconf modules
    
    Your package directory must contain a __init__.py for us to find subsequent packages/modules.
    
    Args:
        paths (list):               list of search path, any iterable will do.
        root_path (str):            Your project root directory path relative to which modules are imported.
        exclude (list, optional):   A list of package names that will be excluded from search.
                                    Must either pass a list or None. None will use internally defined
                                    exclude packages. Defaults to None.
    
    Returns:
        list: List of django URLPatterns
    """

    if exclude is None:
        exclude = ['migrations', 'models',]

    urlpatterns = []

    def _iterpacks(packs, root_path, urlpatterns):
        for loader, name, is_pkg in pkgutil.walk_packages(packs):
            # logger.debug(f"Path: {loader.path} Name: {name} IsPkg: {is_pkg}")
            if is_pkg:
                if name in exclude:
                    logger.debug(f"Package {name} is excluded from search..")
                else:
                    _iterpacks([os.path.join(loader.path, name)], root_path, urlpatterns)
            else:
                if name == DJPUB_URL_CONF_NAME:
                    # logger.debug(f"SEARCH: module {name} matches {DJPUB_URL_CONF_NAME}")
                    full_path = os.path.join(loader.path, name)
                    logger.debug(f"Full path: {full_path}")
                    full_dotted_path = get_dotted_path(full_path, root_path)
                    logger.debug(f"Dotted path: {full_dotted_path}")
                    _URLPatterns_Class = import_string(f"{full_dotted_path}.{UrlPatterns.__name__}")
                    _URLPatterns = _URLPatterns_Class(current_djpub_view_dotted_path=full_dotted_path)
                    _urlpatterns = _URLPatterns._get_urlpatterns()
                    logger.debug(f"UrlPatterns.get_urlpatterns(): {str(_urlpatterns)}")
                    _position = _URLPatterns._get_desired_position_index()
                    urlpatterns[_position:_position] = _urlpatterns
                else:
                    # logger.debug(f"SEARCH: module {name} does not match {DJPUB_URL_CONF_NAME}")
                    pass
    
    _iterpacks(paths, root_path, urlpatterns)
    logger.debug(f"urlpatterns: {str(urlpatterns)}")
    return urlpatterns